#include"Esp8266HwSwTimers.h"


// note!!! gpio 14 and 16 have to connect together.(D0 & D5)
#define ADC_S_PERIOD        191 // 191us
#define INT_GPIO            14
#define TRG_GPIO            16

cHwTimer timer_get_adc_manip;
IRAM_ATTR unsigned timer_trigger=0;
IRAM_ATTR void isr_get_adc_manip_timer(){digitalWrite(TRG_GPIO, timer_trigger^=1);}


unsigned get_adc_data=0; // the current adc value. independently updating once demand.
unsigned get_adc_time_us=0; // how long to do this prob. dec by isr. set nonzero by client to start prob.
unsigned get_adc_timestamp=0; // if event happen, set the timestamp/micros. cleared by client before use.
unsigned get_adc_strict_indec=0; // strictly increasing or decreasing on the demanded interval is required.
unsigned get_adc_v1=2; // set by client the event is v1 transits to v2 in inc/dec manner. v can not <=vbr.
unsigned get_adc_v2=2; // set by client the event is v1 transits to v2 in inc/dec manner. v can not <=vbr.
unsigned get_adc_vbr=1; // tolerance.
unsigned get_adc_got=0;
// for example, get_adc_v1=6, get_adc_v2=10, get_adc_vbr=5, get_adc_time_us=350(but done by 382):
// the event of voltage increasingly transits from 6 to 10 within 350us, if happened within specified time, timestamp set.
// if SMPS=191us, it heavily affects wifi into disappeared. so do the sampling once demand.
// ProbAdcEventTrigger(w, x, y, z); while (!ProbAdcEventPolling() && !timeout) delay(1);
ICACHE_RAM_ATTR void isr_get_adc_manip(){
    ////get_adc_data=analogRead(A0);
    if (get_adc_time_us>=ADC_S_PERIOD){
        get_adc_data=analogRead(A0);////
        get_adc_time_us-=ADC_S_PERIOD;
        if (!get_adc_got){
            if ((get_adc_v1<=get_adc_v2) && (get_adc_data<=(get_adc_v1+get_adc_vbr))) get_adc_got=(get_adc_data-get_adc_vbr);
            else if ((get_adc_v1>get_adc_v2) && (get_adc_data>=(get_adc_v1-get_adc_vbr))) get_adc_got=(get_adc_data+get_adc_vbr);
            else if (get_adc_strict_indec) get_adc_time_us=0;
        }
        else if (get_adc_v1<=get_adc_v2){
            if (get_adc_data<(get_adc_v2-get_adc_vbr)){
                if (get_adc_data>=(get_adc_got-get_adc_vbr)) get_adc_got=(get_adc_data-get_adc_vbr);
                else {get_adc_got=0; if (get_adc_strict_indec) get_adc_time_us=0;}
            }
            else {get_adc_timestamp=micros(); get_adc_got=0; get_adc_time_us=0;}
        }
        else {
            if (get_adc_data>(get_adc_v2+get_adc_vbr)){
                if (get_adc_data<=(get_adc_got+get_adc_vbr)) get_adc_got=(get_adc_data+get_adc_vbr);
                else {get_adc_got=0; if (get_adc_strict_indec) get_adc_time_us=0;}
            }
            else {get_adc_timestamp=micros(); get_adc_got=0; get_adc_time_us=0;}
        }
    }
    else if (get_adc_got){
        get_adc_data=analogRead(A0);////
        if ((get_adc_v1<=get_adc_v2) && (get_adc_data>=(get_adc_v2-get_adc_vbr)) ||\
            (get_adc_v1>get_adc_v2) && (get_adc_data<=(get_adc_v2+get_adc_vbr)))
            get_adc_timestamp=micros();
        get_adc_got=0; get_adc_time_us=0;
    }
    else get_adc_time_us=0;
}

void ProbAdcEventTrigger(unsigned v1, unsigned v2, unsigned tolerance, unsigned timespan_us, bool strict_indec=false){
    get_adc_time_us=0;
    get_adc_got=0;
    get_adc_time_us=0;

    get_adc_strict_indec=strict_indec;
    get_adc_v1=v1;
    get_adc_v2=v2;
    get_adc_vbr=tolerance;
    get_adc_timestamp=0;
    get_adc_time_us=timespan_us;
}

unsigned ProbAdcEventPolling(){
    return get_adc_timestamp;
}


void setup() {
    // put your setup code here, to run once:

    Serial.begin(115200);
    delay(5000);

    pinMode(TRG_GPIO, OUTPUT); // this gpio is to trigger int gpio
    digitalWrite(TRG_GPIO, LOW);
    attachInterrupt(digitalPinToInterrupt(INT_GPIO), isr_get_adc_manip, CHANGE);

    timer_get_adc_manip.setTimer(ADC_S_PERIOD, isr_get_adc_manip_timer);

    // for led
    pinMode(2, OUTPUT); // purpose is adc sampling at most yields to affect nothing.
}

void loop() {
    // put your main code here, to run repeatedly:

    static unsigned tmo1=micros();
    if (cHwTimer::Timeout(10, tmo1)) // probe every 10ms, if adc increases from 100 to 105 within 400us then the Polling shows timestamp.
        ProbAdcEventTrigger(100, 105, 1, 400, true); // note: minimal resolution is 382us.

    static unsigned u=0;
    if (u!=ProbAdcEventPolling()){
        if (ProbAdcEventPolling()){
            u=ProbAdcEventPolling();
            printf("Got the signal ADC rised from 100 to 105 within 400us, at timestamp %u\r\n", u);
        }
    }

    // for led
    static unsigned tmo2=micros(), x=0;
    if (cHwTimer::Timeout(50, tmo2)) digitalWrite(2, x^=1);
}
